/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific language governing rights and
 * limitations under the License.
 * 
 * The Original Code is SomaCore.
 * 
 * The Initial Developer of the Original Code is Romuald Quantin.
 * romu@soundstep.com (www.soundstep.com).
 * 
 * Portions created by
 * 
 * Initial Developer are Copyright (C) 2008-2010 Soundstep. All Rights Reserved.
 * All Rights Reserved.
 * 
 */

package com.soma.core.interfaces {
	import flash.events.IEventDispatcher;

	/**
	 * <p><b>Author:</b> Romuald Quantin - <a href="http://www.soundstep.com/" target="_blank">www.soundstep.com</a></p>
	 * <p><b>Resources:</b> <a href="http://www.soundstep.com/downloads/somacore" target="_blank">http://www.soundstep.com/downloads/somacore</a></p>
	 * <p><b>Actionscript version:</b> 3.0</p>
	 * <p><b>Copyright:</b>
	 * Mozilla Public License 1.1 (MPL 1.1)<br /> 
	 * <a href="http://www.opensource.org/licenses/mozilla1.1.php" target="_blank">http://www.opensource.org/licenses/mozilla1.1.php</a></p>
	 * @see com.soma.core.model.Model
	 */
	
	public interface IModel {
		
		/**
		 * Retrieves the name of the model.
		 * @return A String.
		 */
		function getName():String;
		/**
		 * Sets the name of the model.
		 * @param value A String.
		 */
		function setName(name:String):void;
		/**
		 * Data of the model.
		 */
		function get data():Object;
		function set data(value:Object):void;
		/**
		 * EventDispatcher instance of the model.
		 * @default The framework instance.
		 */
		function get dispatcher():IEventDispatcher;
		function set dispatcher(value:IEventDispatcher):void;
		/**
		 * Method that you can overwrite, called by the framework when the model is initialized.
		 */
		function initialize():void;
		/**
		 * Method that you can overwrite, called by the framework when the model is removed.
		 */
		function dispose():void;
		
	}
}